import 'package:flutter/material.dart';
import 'painter.dart';
import 'enum.dart';

class CustomLinearIndicator extends StatefulWidget {
  const CustomLinearIndicator({
    Key? key,
    this.duration,
    this.height,
    this.value,
    this.width,
    this.filled = false,
    this.backColor,
    this.valueColor,
    this.shadow,
    this.valueTextStyle,
    this.checkOnFinish = false,
    this.finishedAnimation = true,
    this.indicatorType = IndicatorType.inside,
    this.pointerColor,
    this.checkColor,
    this.checkSize,
    this.edgeType = EdgeType.round,
    this.showValue = true,
  }) : super(key: key);

  final double? value, height, width, checkSize;
  final Duration? duration;
  final bool filled;
  final Color? backColor, valueColor, pointerColor, checkColor;
  final BoxShadow? shadow;
  final TextStyle? valueTextStyle;
  final bool checkOnFinish;
  final bool finishedAnimation;
  final IndicatorType indicatorType;
  final EdgeType edgeType;
  final bool showValue;

  @override
  _CustomLinearIndicatorState createState() => _CustomLinearIndicatorState();
}

class _CustomLinearIndicatorState extends State<CustomLinearIndicator>
    with TickerProviderStateMixin {
  AnimationController? widthController;
  AnimationController? scaleController;
  AnimationController? marginController;
  double? value, width, height, checkSize;
  Duration? duration;
  Color? backColor, valueColor, pointerColor, checkColor;
  BoxShadow? shadow;
  TextStyle? valueTextStyle;
  bool filled = false;
  bool animationsInitialized = false, scaleAnimationInitialized = false;
  double factor = 0, textOpacity = 1;
  bool check = false,
      checkOnFinish = false,
      finishedAnimation = false,
      increaseRadius = false;
  double screenWidth = 0, screenHeight = 0;
  IndicatorType indicatorType = IndicatorType.inside;
  bool showValue = true;
  EdgeType edgeType = EdgeType.round;

  void setValues() {
    value = widget.value;
    showValue = widget.showValue;
    edgeType = widget.edgeType;
    indicatorType = widget.indicatorType;
    checkOnFinish = widget.value == null ? false : widget.checkOnFinish;
    finishedAnimation = checkOnFinish ? widget.finishedAnimation : false;
    width = widget.width == null
        ? 0.6 * screenWidth
        : widget.width! <= 0.1 * screenWidth
            ? 0.2 * screenWidth
            : widget.width! > 1 * screenWidth
                ? 1 * screenWidth
                : widget.width;
    height = widget.height ?? 30;
    filled = widget.filled;
    checkColor = widget.checkColor ?? Colors.black;
    checkSize = widget.checkSize ?? height! * 0.6;
    backColor = widget.backColor ?? Colors.black;
    valueColor = widget.valueColor ?? Colors.white;
    pointerColor = widget.pointerColor ?? Colors.black;
    duration = widget.duration ?? const Duration(milliseconds: 500);
    shadow = widget.shadow ??
        BoxShadow(
          color: Colors.black.withOpacity(0.2),
          blurRadius: 6,
          spreadRadius: 1,
        );
    valueTextStyle = widget.valueTextStyle ??
        TextStyle(
          fontSize: (height ?? 30) / 2,
          fontWeight: FontWeight.bold,
        );
    factor = filled
        ? 100
        : width! >= 0.7 * screenWidth
            ? 102.5
            : width! <= 0.7 * screenWidth && width! >= 0.5 * screenWidth
                ? 105
                : width! < 0.3 * screenWidth
                    ? 110
                    : 107.5;
  }

  void initializeAnimations() {
    if (checkOnFinish && !scaleAnimationInitialized) {
      scaleAnimationInitialized = true;
      scaleController = AnimationController(
        vsync: this,
        duration: const Duration(milliseconds: 300),
        lowerBound: 1,
        upperBound: 1.1,
      );

      scaleController!.addListener(() {
        setState(() {});
      });

      scaleController!.addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          scaleController!.reverse();
        } else if (status == AnimationStatus.dismissed) {
          Future.delayed(const Duration(milliseconds: 500)).then((value) {
            scaleController!.forward();
          });
        }
      });
    }
    if (value == null && !animationsInitialized) {
      animationsInitialized = true;
      widthController = AnimationController(
        vsync: this,
        lowerBound: 0.1 * screenWidth,
        upperBound: width!,
        duration: duration,
      );
      marginController = AnimationController(
        vsync: this,
        lowerBound: 0,
        upperBound: width! - 0.1 * screenWidth,
        duration: duration,
      );

      widthController!.addListener(() {
        setState(() {});
      });
      marginController!.addListener(() {
        setState(() {});
      });

      widthController!.addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          marginController!.forward();
        } else if (status == AnimationStatus.dismissed) {
          widthController!.forward();
        }
      });

      marginController!.addStatusListener((status) {
        if (status == AnimationStatus.completed) {
          marginController!.reverse();
        } else if (status == AnimationStatus.dismissed) {
          widthController!.reverse();
        }
      });

      widthController!.forward();
    }
  }

  @override
  void dispose() {
    if (scaleController != null) {
      scaleController!.dispose();
    }
    if (marginController != null) {
      widthController!.dispose();
      marginController!.dispose();
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    screenWidth = MediaQuery.of(context).size.width;
    screenHeight = MediaQuery.of(context).size.height;
    setValues();
    initializeAnimations();
    if (value != null && value! > 100) {
      value = 100;
    }
    if (value == 100 && checkOnFinish && !check) {
      textOpacity = 0;
      increaseRadius = true;
      Future.delayed(const Duration(milliseconds: 500)).then((value) {
        setState(() {
          check = true;
        });
        scaleController!.forward();
      });
    }
    if (value != null && value! < 100) {
      checkOnFinish = false;
      textOpacity = 1;
      check = false;
      increaseRadius = false;
    }
    if (check && !checkOnFinish) {
      textOpacity = 1;
      check = false;
      increaseRadius = false;
    }
    return Center(
      child: SizedBox(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          mainAxisSize: MainAxisSize.min,
          children: [
            value == null || showValue == false
                ? const SizedBox()
                : (value == 100 && checkOnFinish) ||
                        indicatorType == IndicatorType.inside
                    ? SizedBox(
                        height: indicatorType == IndicatorType.inside
                            ? 0
                            : 0.075 * screenWidth,
                      )
                    : Center(
                        child: Container(
                          width: width! + (0.06 * screenWidth),
                          alignment: Alignment.centerLeft,
                          padding: EdgeInsets.only(
                            left: (((width ?? 0.6 * screenWidth) / factor) *
                                value!),
                          ),
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.center,
                            children: [
                              Text(
                                '${(value!.toInt() > 9 ? value!.toInt() : '0${value!.toInt()}')}%',
                                style: TextStyle(
                                  fontWeight: FontWeight.bold,
                                  fontSize: screenWidth * 0.1 / 2.5,
                                ),
                              ),
                              SizedBox(
                                height: screenWidth * 0.075 / 2,
                                width: screenWidth * 0.075,
                                child: CustomPaint(
                                  painter: IndicatorPointer(
                                    color: pointerColor!,
                                  ),
                                  child: Container(),
                                ),
                              ),
                            ],
                          ),
                        ),
                      ),
            // Padding(
            //   padding: EdgeInsets.only(
            //     left: value == null
            //         ? 0
            //         : (((width ?? 0.6 * screenWidth) / factor) * value!),
            //   ),
            //   child: SizedBox(
            //     height: 0.075 * screenWidth,
            //     width: 0.075 * screenWidth,
            //     child: CustomPaint(
            //       painter: IndicatorPointer(),
            //       child: Container(),
            //     ),
            //   ),
            // ),
            SizedBox(
              width: width ?? 0.6 * screenWidth,
              height: height ?? 30,
              child: Stack(
                children: [
                  Center(
                    child: Transform.scale(
                      scale: checkOnFinish && finishedAnimation
                          ? scaleController == null
                              ? 1
                              : scaleController!.value
                          : 1,
                      child: AnimatedContainer(
                        duration: const Duration(milliseconds: 500),
                        alignment: Alignment.centerLeft,
                        width: value == 100 && checkOnFinish
                            ? height
                            : (width ?? 0.6 * screenWidth),
                        height: height ?? 30,
                        clipBehavior: Clip.hardEdge,
                        padding: filled ? null : const EdgeInsets.all(5),
                        decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(
                            edgeType == EdgeType.round ? height! : 0,
                          ),
                          color: backColor,
                          boxShadow: [
                            shadow!,
                          ],
                        ),
                        child: Container(
                          margin: EdgeInsets.only(
                            left: value == null
                                ? marginController == null
                                    ? 0
                                    : marginController!.value
                                : 0,
                          ),
                          height: height,
                          width: value == null
                              ? widthController == null
                                  ? 0
                                  : widthController!.value
                              : (((width ?? 0.6 * screenWidth) / factor) *
                                  value!),
                          decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(
                              edgeType == EdgeType.round ? height! : 0,
                            ),
                            color: valueColor,
                          ),
                        ),
                      ),
                    ),
                  ),
                  value == null || showValue == false
                      ? const SizedBox()
                      : check && checkOnFinish
                          ? Center(
                              child: Icon(
                                Icons.check,
                                size: checkSize,
                                color: checkColor,
                              ),
                            )
                          : indicatorType != IndicatorType.inside
                              ? const SizedBox()
                              : AnimatedOpacity(
                                  opacity: textOpacity,
                                  duration: const Duration(milliseconds: 400),
                                  child: Center(
                                    child: Text(
                                      '${(value!.toInt() > 9 ? value!.toInt() : '0${value!.toInt()}')}%',
                                      style: valueTextStyle!,
                                    ),
                                  ),
                                ),
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }
}
