import 'package:flutter/material.dart';
import 'dart:math' as math;

class IndicatorPointer extends CustomPainter {
  final Color color;
  IndicatorPointer({
    required this.color,
  });

  @override
  void paint(Canvas canvas, Size size) {
    double width = size.width;
    double totalHeight = size.height;
    Paint paint = Paint()
      ..color = color
      ..style = PaintingStyle.fill
      ..strokeCap = StrokeCap.round
      ..strokeWidth = 4;

    Path path = Path();

    path.addPolygon([
      Offset(width * 0.35, 0),
      Offset(width * 0.65, 0),
      Offset(width * 0.5, totalHeight),
    ], true);
    path.close();
    path.close();

    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}


class DrawCircles extends CustomPainter {
  final double firstCircleValue, secondCircleValue;
  final Color backColor, valueColor;

  DrawCircles({
    required this.firstCircleValue,
    required this.secondCircleValue,
    required this.backColor,
    required this.valueColor,
  });

  @override
  void paint(Canvas canvas, Size size) {
    double width = size.width, height = size.height;
    Paint backCirclePaint = Paint()
      ..color = backColor
      ..strokeWidth = width * 0.14
      ..strokeCap = StrokeCap.round
      ..style = PaintingStyle.stroke;
    Paint firstCirclePaint = Paint()
      ..color = valueColor
      ..strokeWidth = width * 0.05
      ..strokeCap = StrokeCap.round
      ..style = PaintingStyle.stroke;
    Paint secondCirclePaint = Paint()
      ..color = backColor
      ..strokeWidth = width * 0.05
      ..strokeCap = StrokeCap.round
      ..style = PaintingStyle.stroke;

    Path backCirclePath = Path();
    Path firstCirclePath = Path();
    Path secondCirclePath = Path();
    backCirclePath.addArc(
      Rect.fromCenter(
        center: Offset(width * 0.5, height * 0.5),
        width: width,
        height: height,
      ),
      0,
      math.pi * 2,
    );
    firstCirclePath.addArc(
      Rect.fromCenter(
        center: Offset(width * 0.5, height * 0.5),
        width: width,
        height: height,
      ),
      0,
      firstCircleValue,
    );
    secondCirclePath.addArc(
      Rect.fromCenter(
        center: Offset(width * 0.5, height * 0.5),
        width: width,
        height: height,
      ),
      0,
      secondCircleValue,
    );
    canvas.drawPath(backCirclePath, backCirclePaint);
    canvas.drawPath(firstCirclePath, firstCirclePaint);
    canvas.drawPath(secondCirclePath, secondCirclePaint);
    // canvas.drawPath(path, paint);
    // canvas.drawCircle(Offset(width * 0.5, height * 0.5), height * 0.5, paint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}